package com.kedacom.musicplayer;

import android.app.Service;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.Intent;
import android.media.AudioAttributes;
import android.media.MediaPlayer;
import android.net.Uri;
import android.os.Binder;
import android.os.IBinder;
import android.util.Log;

import java.io.IOException;
import java.util.List;

public class PlayerService extends Service implements
        MediaPlayer.OnPreparedListener,
        MediaPlayer.OnErrorListener,
        MediaPlayer.OnCompletionListener {

    private final static String TAG = "PlayerService";
    private final IBinder playerBind = new PlayerServiceBinder();
    private MediaPlayer mediaPlayer;
    private Integer resumePosition;
    public Uri mediaFileUri;

    public List<String> currentPlayList;
    public Integer currentPlayIndex = 0;

    private BroadcastReceiver pauseBroadcastReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            Log.d(TAG, "onReceive: pause");
            pause();
        }
    };

    private BroadcastReceiver resumeBroadcastReceiver = new BroadcastReceiver() {
        @Override
        public void onReceive(Context context, Intent intent) {
            Log.d(TAG, "onReceive: resume");
            resume();
        }
    };

    public class PlayerServiceBinder extends Binder {
        public PlayerService getService() {
            return PlayerService.this;
        }
    }

    private void playerInit() {
        Log.d(TAG, "playerInit: init command issued...");
        mediaPlayer.setOnPreparedListener(PlayerService.this);
        mediaPlayer.setOnErrorListener(PlayerService.this);
        mediaPlayer.setOnCompletionListener(PlayerService.this);
        AudioAttributes audioAttributes = new AudioAttributes.Builder()
                .setContentType(AudioAttributes.CONTENT_TYPE_MUSIC)
                .setUsage(AudioAttributes.USAGE_MEDIA)
                .build();
        mediaPlayer.setAudioAttributes(audioAttributes);
        Log.d(TAG, "playerInit: ok");
    }

    private Boolean prepareNewTrack(String filePath) {
        Log.d(TAG, "prepareNewTrack: setting new track...");
        try {
            if (filePath == null || filePath.equals("")) {
                Log.d(TAG, "prepareNewTrack: empty file path, do nothing...");
                return false;
            }
            mediaFileUri = Uri.parse(filePath);
            mediaPlayer.setDataSource(this, mediaFileUri);
//            mediaPlayer.prepareAsync();
            mediaPlayer.prepare();
        } catch (IOException e) {
            Log.d(TAG, "prepareNewTrack: failed, try next one...");
            if ((++currentPlayIndex) < currentPlayList.size())
                prepareNewTrack(currentPlayList.get(currentPlayIndex));
            else
                return false;
        }
        Log.d(TAG, "prepareNewTrack: done preparing");
        return true;
    }


    @Override
    public void onCreate() {
        super.onCreate();
        mediaPlayer = new MediaPlayer();
        resumePosition = 0;
        Log.d(TAG, "onCreate: ok");
    }

    @Override
    public IBinder onBind(Intent intent) {
        Log.d(TAG, "onBind: working");
        return playerBind;
    }

    @Override
    public boolean onUnbind(Intent intent) {
        Log.d(TAG, "onUnbind: ok");
        return false;
    }


    @Override
    public void onDestroy() {
        super.onDestroy();
        if (mediaFileUri != null) {
            stop();
            mediaPlayer.release();
        }
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        playerInit();
        Log.d(TAG, "onStartCommand: done");
        return super.onStartCommand(intent, flags, startId);
    }


    @Override
    public void onPrepared(MediaPlayer mediaPlayer) {
        Log.d(TAG, "onPrepared: done preparing");
//        play();
    }


    @Override
    public boolean onError(MediaPlayer mediaPlayer, int i, int i1) {
        switch (i) {
            case MediaPlayer.MEDIA_ERROR_NOT_VALID_FOR_PROGRESSIVE_PLAYBACK:
                Log.d(TAG, "onError: not valid for progressive playback. " + i1);
                break;
            case MediaPlayer.MEDIA_ERROR_SERVER_DIED:
                Log.d(TAG, "onError: server died. " + i1);
                break;
            case MediaPlayer.MEDIA_ERROR_UNKNOWN:
                Log.d(TAG, "onError: unknonw error. " + i1);
                break;
        }
        return false;
    }

    @Override
    public void onCompletion(MediaPlayer mediaPlayer) {
        stop();
        if (currentPlayList == null
                || currentPlayList.isEmpty()
                || currentPlayIndex == (currentPlayList.size() - 1)) {
            return;
        } else {
            next();
            Log.d(TAG, "onCompletion: playing next: " + currentPlayList.get(currentPlayIndex));
        }
    }


    public void play(String path) {
        stop();
        mediaPlayer.reset();
        if (prepareNewTrack(path)) {
            mediaPlayer.start();
            resumePosition = 0;
            sendServiceBroadcast(MainActivity.BROADCAST_PLAYING_START_NEW);
            Log.d(TAG, "play: playing: " + path);
        } else {
            Log.d(TAG, "play: exiting...");
        }
    }

    public void next() {
        if (currentPlayList != null
                && !currentPlayList.isEmpty()
                && currentPlayIndex < (currentPlayList.size() - 1))
            play(currentPlayList.get((++currentPlayIndex)));
        else
            // list ends.
            Log.d(TAG, "next: list ends");

    }

    public void previous() {
        if (currentPlayList != null
                && !currentPlayList.isEmpty()
                && currentPlayIndex > 0)
            play(currentPlayList.get((--currentPlayIndex)));
        else
            Log.d(TAG, "previous: list at 0 index`");
    }

    public void stop() {
        Log.d(TAG, "stop: stop command issued...");
        if (mediaPlayer == null) {
            Log.d(TAG, "stop: null mediaPlayer instance!");
            return;
        }
        if (mediaPlayer.isPlaying()) {
            mediaPlayer.stop();
            sendServiceBroadcast(MainActivity.BROADCAST_PLAYING_END);
        }
        Log.d(TAG, "stop: stopped");
    }

    public void pause() {
        Log.d(TAG, "pause: pause command issued...");
        if (mediaPlayer != null && mediaPlayer.isPlaying()) {
            mediaPlayer.pause();
            resumePosition = mediaPlayer.getCurrentPosition();
            Log.d(TAG, "pause: position saved, paused");
        }
    }

    public void resume() {
        Log.d(TAG, "resume: resuem command issued...");
        if (resumePosition != 0 && !mediaPlayer.isPlaying()) {
            mediaPlayer.seekTo(resumePosition);
            mediaPlayer.start();
            Log.d(TAG, "resume: resuming from saved position...");
        }
    }

    public void sendServiceBroadcast(String action) {
        Intent intent = new Intent(action);
        sendBroadcast(intent);
    }

    public Boolean isPlaying() {
        return mediaPlayer.isPlaying();
    }

    public Boolean isPaused() {
        if (resumePosition == null || resumePosition.equals(0))
            return false;
        else
            return true;
    }

    public Integer getCurrentPosition() {
        return mediaPlayer.getCurrentPosition();
    }

    public Integer getCurrentDuration() {
        return mediaPlayer.getDuration();
    }

    public Integer getCurrentPlayIndex() {
        return currentPlayIndex;
    }
}
